clc;
close all;
clear all;

L=2;
T=1;

SNR_vec=[0:5:100];
Avg=100;
mse_bspline=zeros(size(SNR_vec));
mse_espline=zeros(size(SNR_vec));
mse_ours=zeros(size(SNR_vec));


% splines param
spline_order = 2*L-1;
S = spline_order+1; % spline support

% espline init
switch L
    case 2
        alpha_vec=0.25*[0 - 2.3562i        0 - 0.7854i        0 + 0.7854i        0 + 2.3562i];
    case 4
        alpha_vec=0.12*[0 - 2.7489i        0 - 1.9635i        0 - 1.1781i        0 - 0.3927i,... 
        0 + 0.3927i        0 + 1.1781i        0 + 1.9635i        0 + 2.7489i];
    otherwise
        error('only L=2 or 4');
end

    
% number of samples
N=2*L*S;
ts=T/N;
Ts_grid=((0:N-1)*ts)';

% delays
switch L
    case 2
        tl=[0.256 0.38];
        al=[1 0.8];
    case 4
        tl=[0.213 0.452 0.664 0.7453];
        al=[1 0.9 0.7 0.6];
    otherwise
        error('only L=2 or 4');
end

% multi fri param
p=N-1;
M=floor(p/2);
Smix=[eye(M) zeros(M,1) fliplr(eye(M));...
    zeros(1,M) sqrt(2) zeros(1,M);...
    -j*eye(M) zeros(M,1)  j*fliplr(eye(M))];
   

%% B-Spline Sampling
% coeff calc
spline_coefficients = zeros(N,spline_order+1);
for i_moment = 1:S
    spline_coefficients(:,i_moment) = CalcSplineCoeff(spline_order,i_moment-1,N);
end

% sampling
tl_b=ts*(N/2-1)+tl-round(mean(tl)/ts)*ts;
c_bspline=zeros(N,1);

figure(1);
stem(tl_b,al);
hold on;
for i=1:L
    c_bspline = c_bspline+calcspline((Ts_grid-tl_b(i))/ts,spline_order)*al(i);
    plot(Ts_grid,calcspline((Ts_grid-tl_b(i))/ts,spline_order)*al(i),'r');
end
hold off;

optimal_samples_bspline=(c_bspline>10*eps);
Ec_bspline=sum(c_bspline.^2)/length(optimal_samples_bspline);

%% E-Spline Sampling
% coeff calc
Cmat=CalcEsplineCoeffMat(spline_order,alpha_vec,N);

df=(alpha_vec(2)-alpha_vec(1))/j;
res=ts/(df/(2*pi));

espline = CalcEspline(alpha_vec);

c_espline=zeros(N,1);
figure(2);
hold on;
stem(tl,al);
for i=1:L
    t=(Ts_grid-tl(i))/ts+10*eps;
    espline_values=subs(espline).*(t>=0 & t<=S);
    c_espline=c_espline+espline_values*al(i);
    plot(Ts_grid,espline_values*al(i),'r');
end
hold off;

optimal_samples_espline=(c_espline>10*eps);
Ec_espline=sum(c_espline.^2)/length(optimal_samples_espline);


%%  multiFRI sampling
c=MultiSampMix(tl,al,Smix,T); % tones
Ec=sum(abs(c).^2)/p;


for snr_ind=1:length(SNR_vec)
    for avg_ind=1:Avg
        
        SNR=SNR_vec(snr_ind);

        % B-Spline recovery
        nb=randn(size(c_bspline))*10^(-SNR/20)*sqrt(Ec_bspline);
        cn_bspline=c_bspline+nb;
        moments=sum(spline_coefficients.*repmat(cn_bspline.*optimal_samples_bspline,[1 S]),1);
        tl_bspline=AnnFilterBspline(moments,L,T,ts)+ts*(N/2);
         mse_bspline(snr_ind)=mse_bspline(snr_ind)+1/L/Avg*sum((tl_bspline-tl_b').^2);
                  
         % E-Spline recovery
         ne=randn(size(c_espline))*10^(-SNR/20)*sqrt(Ec_espline);
         moments=Cmat*((c_espline+ne).*optimal_samples_espline);
         tl_espline=AnnFilterEspline(moments.',L,T,ts,df);
         mse_espline(snr_ind)=mse_espline(snr_ind)+1/L/Avg*sum((tl_espline-tl').^2);
         
         % Multi recovery
          n=randn(size(c))*10^(-SNR/20)*sqrt(Ec);
          cn=c+n;
          x=pinv(Smix)*cn;
          tl_est=KT_fb(x.',floor(2/3*p),L);
           mse_ours(snr_ind)=mse_ours(snr_ind)+1/L/Avg*sum((tl_est-tl').^2);
           
      end
end

figure;
hold on;
plot(SNR_vec,10*log10(mse_ours),'g','LineWidth',1.5);
plot(SNR_vec,10*log10(mse_espline),'c--','LineWidth',1.5);
plot(SNR_vec,10*log10(mse_bspline),'r:','LineWidth',1.5);
xlabel('SNR [dB]');
ylabel('MSE [dB]');
grid on;
legend('tones','E-spline','B-spline','location','SouthWest');
